package com.jwl.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.dto.QuestionQueryDto;
import com.jwl.driver.server.entity.TdQuestion;
import com.jwl.driver.server.entity.TdSysConfigList;
import com.jwl.driver.server.mapper.TdQuestionMapper;
import com.jwl.driver.server.service.ITdQuestionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.service.ITdSysConfigListService;
import com.jwl.driver.server.service.ITdSysConfigService;
import com.jwl.driver.server.vo.QusetionCategoryVo;
import com.jwl.driver.server.vo.QusetionVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 题库; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class TdQuestionServiceImpl extends ServiceImpl<TdQuestionMapper, TdQuestion> implements ITdQuestionService {

    @Autowired
    private ITdSysConfigService configService;

    @Autowired
    private ITdSysConfigListService configListService;

    @Override
    public TdQuestion queryQuestionById(QuestionQueryDto queryDto) {
        return this.getBaseMapper().selectById(queryDto.getQuestionId());
    }

    @Override
    public List<TdQuestion> queryQuestionByIdList(QuestionQueryDto queryDto) {
        LambdaQueryWrapper<TdQuestion> queryWrapper = new LambdaQueryWrapper<TdQuestion>()
                .in(TdQuestion::getQuestionId,queryDto.getQuestionIdList());

        return this.getBaseMapper().selectList(queryWrapper);
    }

    @Override
    public List<QusetionVo> queryQuestion(QuestionQueryDto queryDto) {
        return this.getBaseMapper().queryQuestion(queryDto);
    }

    @Override
    public List<QusetionVo> getTestQuestion(QuestionQueryDto queryDto) {
        List<QusetionVo> resultList = new ArrayList<>();

        if (StrUtil.equals(Constants.SUBJECT_ONE,queryDto.getSubject())){
            //40道判断题 60道单选 每题1分
            queryDto.setType(Constants.QUESTION_TYPE_ONE)
                    .setNum(60);
            resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));

            queryDto.setType(Constants.QUESTION_TYPE_TWO)
                    .setNum(40);
            resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));

        }else if(StrUtil.equals(Constants.SUBJECT_FOUR,queryDto.getSubject())){
            //20道判断题 20道单选 10道多选，每题2分，如果各车型不统一 可以配在数据库里
            queryDto.setType(Constants.QUESTION_TYPE_TWO)
                    .setNum(20);

            resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            queryDto.setType(Constants.QUESTION_TYPE_ONE)
                    .setNum(20);

            resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            queryDto.setType(Constants.QUESTION_TYPE_THREE)
                    .setNum(10);
            resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
        }
        return resultList;
    }

    @Override
    public List<QusetionCategoryVo> questionCategory(QuestionQueryDto queryDto) {

        List<QusetionCategoryVo> resultList = new ArrayList<>();
        //获取题目
        List<TdQuestion> tdQuestions = queryQuestionByIdList(queryDto);
        if (CollectionUtil.isEmpty(tdQuestions)){
            return resultList;
        }
        //获取分类
        List<TdSysConfigList> categoryList = configListService.querySysConfigList(Constants.QUESTION_CATEGORY, Constants.DEFAULT_CARTYPE_ID);
        Map<String, String> categoryMap = categoryList.stream().collect(Collectors.toMap(TdSysConfigList::getConfigItemCode, TdSysConfigList::getConfigItemName, (v1, v2) -> v1));
        Map<String,Integer> totalMap = new HashMap<>();

        for (TdQuestion tdQuestion : tdQuestions) {
            String category = tdQuestion.getCategory();
            if (StrUtil.isBlank(category)){
                continue;
            }
            String[] split = category.split(",");
            for (String s : split) { //遍历字符串数组
                if (totalMap.containsKey(s)) { //判断Map集合中是否有该字符
                    Integer value = totalMap.get(s); //通过key找出集合中的value
                    totalMap.put(s, value + 1); //将值的数据加1，然后添加到集合中去
                } else {
                    totalMap.put(s,1); //此时集合中没有该Key,所以将该字符作为键加入到集合中
                }
            }
        }

        for (String category : totalMap.keySet()) {
            QusetionCategoryVo categoryVo = new QusetionCategoryVo()
                    .setCategory(category)
                    .setCategoryName(categoryMap.getOrDefault(category,"其他类型"))
                    .setNum(totalMap.get(category));
            resultList.add(categoryVo);
        }
        return resultList;
    }

    @Override
    public int updateQuestion(QusetionVo qusetionVo) {
        return this.getBaseMapper().updateQuestion(qusetionVo);
    }
}
